@@ -1,4 +1,4 @@
-# Copyright (c) 2020, DjaoDjin inc.
+# Copyright (c) 2021, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,9 +33,10 @@ information (street_address, etc.) have not been granted to the requesting user.
 The OrganizationDetailSerializer is used when the requesting user has been
 granted access to personal information.
 """
-
 from __future__ import unicode_literals
 
+import logging
+
 from django.core import validators
 from django.contrib.auth import get_user_model
 from django.contrib.auth.hashers import is_password_usable
@@ -43,10 +44,11 @@ from django.db import transaction
 from django.template.defaultfilters import slugify
 from django.utils.translation import ugettext_lazy as _
 from django.urls.exceptions import NoReverseMatch
+from django_countries.serializer_fields import CountryField
 from rest_framework import serializers
 from rest_framework.exceptions import ValidationError
 from rest_framework.generics import get_object_or_404
-from django_countries.serializer_fields import CountryField
+import phonenumbers
 
 from .. import settings
 from ..compat import reverse, six
@@ -59,6 +61,9 @@ from ..utils import (build_absolute_uri, get_organization_model, get_role_model)
 
 #pylint: disable=no-init
 
+LOGGER = logging.getLogger(__name__)
+
+
 class EnumField(serializers.Field):
     """
     Treat a ``PositiveSmallIntegerField`` as an enum.
@@ -72,11 +77,12 @@ class EnumField(serializers.Field):
             slugify(val): key for key, val in six.iteritems(self.choices)}
         super(EnumField, self).__init__(*args, **kwargs)
 
-    def to_representation(self, obj):
-        if isinstance(obj, list):
-            result = [slugify(self.choices.get(item, None)) for item in obj]
+    def to_representation(self, value):
+        if isinstance(value, list):
+            result = [slugify(self.choices.get(item, None))
+                for item in value]
         else:
-            result = slugify(self.choices.get(obj, None))
+            result = slugify(self.choices.get(value, None))
         return result
 
     def to_internal_value(self, data):
@@ -89,20 +95,46 @@ class EnumField(serializers.Field):
                 raise ValidationError(_("This field cannot be blank."))
             raise ValidationError(_("'%(data)s' is not a valid choice."\
                 " Expected one of %(choices)s.") % {
-                    'data': data, 'choices': [choice
+                    'data': data, 'choices': [str(choice)
                     for choice in six.iterkeys(self.inverted_choices)]})
         return result
 
 
+class PhoneField(serializers.Field):
+
+    def to_representation(self, value):
+        return str(value)
+
+    def to_internal_value(self, data):
+        """
+        Returns a formatted phone number as a string.
+        """
+        if self.required:
+            try:
+                phone_number = phonenumbers.parse(data, None)
+            except phonenumbers.NumberParseException as err:
+                LOGGER.info("tel %s:%s", data, err)
+                phone_number = None
+            if not phone_number:
+                try:
+                    phone_number = phonenumbers.parse(data, "US")
+                except phonenumbers.NumberParseException:
+                    raise ValidationError(self.error_messages['invalid'])
+            if phone_number and not phonenumbers.is_valid_number(phone_number):
+                raise ValidationError(self.error_messages['invalid'])
+            return phonenumbers.format_number(
+                phone_number, phonenumbers.PhoneNumberFormat.E164)
+        return None
+
+
 class PlanRelatedField(serializers.RelatedField):
 
     def __init__(self, **kwargs):
         super(PlanRelatedField, self).__init__(
             queryset=Plan.objects.all(), **kwargs)
 
-    # Django REST Framework 3.0
-    def to_representation(self, obj):
-        return obj.slug
+    def to_representation(self, value):
+        return value.slug
 
     def to_internal_value(self, data):
         return get_object_or_404(Plan.objects.all(), slug=data)
@@ -111,11 +143,11 @@ class PlanRelatedField(serializers.RelatedField):
 class RoleDescriptionRelatedField(serializers.RelatedField):
 
     def __init__(self, **kwargs):
-        super(RoleDescriptionRelatedField, self).__init__(**kwargs)
+        super(RoleDescriptionRelatedField, self).__init__(
+            queryset=RoleDescription.objects.all(), **kwargs)
 
-    # Django REST Framework 3.0
-    def to_representation(self, obj):
-        return obj.slug
+    def to_representation(self, value):
+        return value.slug
 
     def to_internal_value(self, data):
         return get_object_or_404(RoleDescription.objects.all(), slug=data)
@@ -193,6 +225,8 @@ class ChargeSerializer(serializers.ModelSerializer):
         help_text=_("Current state (i.e. created, done, failed, disputed)"))
     readable_amount = serializers.SerializerMethodField(
         help_text=_("Amount and unit in a commonly accepted readable format"))
+    detail = serializers.CharField(read_only=True, required=False,
+        help_text=_("Feedback for the user in plain text"))
 
     @staticmethod
     def get_readable_amount(charge):
@@ -201,7 +235,9 @@ class ChargeSerializer(serializers.ModelSerializer):
     class Meta:
         model = Charge
         fields = ('created_at', 'amount', 'unit', 'readable_amount',
-                  'description', 'last4', 'exp_date', 'processor_key', 'state')
+                  'description', 'last4', 'exp_date', 'processor_key', 'state',
+                  'detail')
+        read_only_fields = ('detail',)
 
 
 class CouponCreateSerializer(serializers.ModelSerializer):
@@ -212,7 +248,8 @@ class CouponCreateSerializer(serializers.ModelSerializer):
         help_text=_("Type of discount (percentage or currency unit)"))
     plan = PlanRelatedField(required=False, allow_null=True)
 
-    def validate_plan(self, plan):
+    @staticmethod
+    def validate_plan(plan):
         if plan and not plan.is_active:
             raise ValidationError(_("The plan is inactive. "\
                 "As a result the coupon will have no effect."))
@@ -244,6 +281,8 @@ class EmailChargeReceiptSerializer(NoModelSerializer):
             " parameter)"))
     email = serializers.EmailField(read_only=True,
         help_text=_("E-mail address to which the receipt was sent."))
+    detail = serializers.CharField(read_only=True,
+        help_text=_("Feedback for the user in plain text"))
 
 
 class ForceSerializer(NoModelSerializer):
@@ -339,8 +378,8 @@ class OrganizationSerializer(serializers.ModelSerializer):
         return 'organization'
 
 OrganizationSerializer._declared_fields["type"] = \
-    serializers.SerializerMethodField(
-        help_text=_("One of 'organization', 'personal' or 'user'"))
+    serializers.SerializerMethodField(#pylint:disable=protected-access
+    help_text=_("One of 'organization', 'personal' or 'user'"))
 
 
 class OrganizationDetailSerializer(OrganizationSerializer):
@@ -401,23 +440,24 @@ class OrganizationCreateSerializer(NoModelSerializer):
     extra = serializers.CharField(required=False, allow_null=True,
         help_text=_("Extra meta data (can be stringify JSON)"))
 
-    def validate(self, data):
+    def validate(self, attrs):
         # XXX This is because we use `OrganizationCreateSerializer`
         # in `SubscriptionCreateSerializer`.
-        if not (data.get('slug') or (
-            data.get('full_name') and data.get('email') and data.get('type'))):
+        if not (attrs.get('slug') or (attrs.get('full_name') and
+                attrs.get('email') and attrs.get('type'))):
             raise ValidationError(_("One of slug or (full_name, email,"\
                 " type) should be present"))
-        return super(OrganizationCreateSerializer, self).validate(data)
+        return super(OrganizationCreateSerializer, self).validate(attrs)
 
-    def validate_type(self, value):
+    @staticmethod
+    def validate_type(value):
         if value not in ('personal', 'organization'):
             raise ValidationError(
                 _("type must be one of 'personal' or 'organization'."))
         return value
 
 OrganizationCreateSerializer._declared_fields["type"] = \
-    serializers.CharField(required=False,
+    serializers.CharField(required=False,#pylint:disable=protected-access
         help_text=_("One of 'organization', 'personal' or 'user'"))
 
 
@@ -511,8 +551,11 @@ class PlanSerializer(serializers.ModelSerializer):
         " to the plan initiated by a subscriber. (defaults to False)"))
     advance_discounts = AdvanceDiscountSerializer(many=True, required=False,
         help_text=_("Discounts when periods are paid in advance."))
+
     discounted_period_amount = serializers.SerializerMethodField(required=False,
         help_text=_("Discounted amount for the first period"))
+    is_cart_item = serializers.SerializerMethodField(required=False,
+        help_text=_("The plan is part of the cart to checkout"))
 
     class Meta:
         model = Plan
@@ -522,13 +565,18 @@ class PlanSerializer(serializers.ModelSerializer):
                   'period_length', 'renewal_type', 'is_not_priced',
                   'created_at',
                   'skip_optin_on_grant', 'optin_on_request',
-                  'discounted_period_amount')
-        read_only_fields = ('slug', 'app_url')
+                  'discounted_period_amount', 'is_cart_item')
+        read_only_fields = ('slug', 'app_url',
+            'discounted_period_amount', 'is_cart_item')
 
     @staticmethod
     def get_discounted_period_amount(obj):
         return getattr(obj, 'discounted_period_amount', obj.period_amount)
 
+    @staticmethod
+    def get_is_cart_item(obj):
+        return getattr(obj, 'is_cart_item', False)
+
     @staticmethod
     def get_app_url(obj):
         return product_url(obj.organization)
@@ -629,7 +677,7 @@ class TransactionSerializer(serializers.ModelSerializer):
         help_text=_("True if the transaction is indentified as a debit in"\
         " the API context"))
 
-    def _is_debit(self, transaction):
+    def _is_debit(self, value):
         """
         True if the transaction can be tagged as a debit. That is
         it is either payable by the organization or the transaction
@@ -638,18 +686,18 @@ class TransactionSerializer(serializers.ModelSerializer):
         #pylint: disable=no-member
         if ('view' in self.context
             and hasattr(self.context['view'], 'organization')):
-            return transaction.is_debit(self.context['view'].organization)
+            return value.is_debit(self.context['view'].organization)
         return False
 
-    def to_representation(self, obj):
-        ret = super(TransactionSerializer, self).to_representation(obj)
-        is_debit = self._is_debit(obj)
+    def to_representation(self, instance):
+        ret = super(TransactionSerializer, self).to_representation(instance)
+        is_debit = self._is_debit(instance)
         if is_debit:
-            amount = as_money(obj.orig_amount, '-%s' % obj.orig_unit)
+            amount = as_money(instance.orig_amount, '-%s' % instance.orig_unit)
         else:
-            amount = as_money(obj.dest_amount, obj.dest_unit)
+            amount = as_money(instance.dest_amount, instance.dest_unit)
         ret.update({
-            'description': as_html_description(obj),
+            'description': as_html_description(instance),
             'is_debit': is_debit,
             'amount': amount})
         return ret
@@ -820,14 +868,6 @@ class UploadBlobSerializer(NoModelSerializer):
         help_text=_("URL to uploaded content"))
 
 
-class ValidationErrorSerializer(NoModelSerializer):
-    """
-    Details on why token is invalid.
-    """
-    detail = serializers.CharField(help_text=_("Describes the reason for"\
-        " the error in plain text"))
-
-
 class AgreementSignSerializer(NoModelSerializer):
     read_terms = serializers.BooleanField(help_text=_(
         "I have read and understand these terms and conditions"))
@@ -840,3 +880,36 @@ class CreateAccessibleRequestSerializer(NoModelSerializer):
     """
     organization = serializers.CharField()
     message = serializers.CharField(max_length=255, required=False)
+
+
+class LifetimeSerializer(OrganizationSerializer):
+    """
+    A customer lifetime value
+    """
+    created_at = serializers.CharField(
+        help_text=_("Since when is the profile a subscriber"))
+    ends_at = serializers.CharField(
+        help_text=_("Current end date for the contract"))
+    contract_value = serializers.IntegerField(
+        help_text=_("Total value to be collected from the profile"))
+    cash_payments = serializers.IntegerField(
+        help_text=_("Cash payments collected from the profile"))
+    deferred_revenue = serializers.IntegerField(
+        help_text=_("The deferred revenue for the profile"))
+    unit = serializers.CharField(
+        help_text=_("Three-letter ISO 4217 code for currency unit (ex: usd)"))
+
+    class Meta(OrganizationSerializer.Meta):
+        fields = OrganizationSerializer.Meta.fields + (
+            'created_at', 'ends_at', 'contract_value',
+            'cash_payments', 'deferred_revenue', 'unit')
+        read_only_fields = ('created_at', 'ends_at', 'contract_value',
+            'cash_payments', 'deferred_revenue', 'unit')
+
+
+class ValidationErrorSerializer(NoModelSerializer):
+    """
+    Details on why token is invalid.
+    """
+    detail = serializers.CharField(help_text=_("Describes the reason for"\
+        " the error in plain text"))
