@@ -1,4 +1,4 @@
-# Copyright (c) 2019, DjaoDjin inc.
+# Copyright (c) 2021, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -98,6 +98,7 @@ _SETTINGS = {
     'EXPIRE_NOTICE_DAYS': [15],
     'EXTRA_MIXIN': object,
     'EXTRA_FIELD': None,
+    'MAX_RENEWAL_ATTEMPTS': 3,
     'MAX_TYPEAHEAD_CANDIDATES': 5,
     'ORGANIZATION_MODEL': 'saas.Organization',
     'PAGE_SIZE': 25,
@@ -113,6 +114,7 @@ _SETTINGS = {
         'PRIV_KEY': None,
         'PUB_KEY': None,
         'REDIRECT_CALLABLE': None,
+        'USE_STRIPE_V3': False,
         'WEBHOOK_URL': 'stripe/postevent',
         'WEBHOOK_SECRET': None,
     },
@@ -143,6 +145,7 @@ DISABLE_UPDATES = _SETTINGS.get('DISABLE_UPDATES')
 EXPIRE_NOTICE_DAYS = _SETTINGS.get('EXPIRE_NOTICE_DAYS')
 EXTRA_MIXIN = _SETTINGS.get('EXTRA_MIXIN')
 IS_BROKER_CALLABLE = _SETTINGS.get('BROKER').get('IS_INSTANCE_CALLABLE', None)
+MAX_RENEWAL_ATTEMPTS = _SETTINGS.get('MAX_RENEWAL_ATTEMPTS')
 MAX_TYPEAHEAD_CANDIDATES = _SETTINGS.get('MAX_TYPEAHEAD_CANDIDATES')
 ORGANIZATION_MODEL = _SETTINGS.get('ORGANIZATION_MODEL')
 PAGE_SIZE = _SETTINGS.get('PAGE_SIZE')
@@ -167,14 +170,4 @@ MANAGER = 'manager'
 CONTRIBUTOR = 'contributor'
 
 PICTURE_STORAGE_CALLABLE = _SETTINGS.get('PICTURE_STORAGE_CALLABLE')
-
-
-def get_extra_field_class():
-    extra_class = _SETTINGS.get('EXTRA_FIELD')
-    if extra_class is None:
-        from django.db.models import TextField
-        extra_class = TextField
-    elif isinstance(extra_class, str):
-        from saas.compat import import_string
-        extra_class = import_string(extra_class)
-    return extra_class
+EXTRA_FIELD = _SETTINGS.get('EXTRA_FIELD')
