@@ -73,7 +73,7 @@
             var country = countryElement.val();
             if( country === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Country");
+                errorMessages += self.options.countryLabel;
                 countryElement.parents(".form-group").addClass("has-error");
                 valid = false;
             }
@@ -83,7 +83,7 @@
             var accountNumber = accountNumberElement.val();
             if(!Stripe.bankAccount.validateAccountNumber(accountNumber, country)) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Account Number");
+                errorMessages += self.options.accountNumberLabel;
                 accountNumberElement.parents(".form-group").addClass("has-error");
                 valid = false;
             }
@@ -91,7 +91,7 @@
             var routingNumber = routingNumberElement.val();
             if(!Stripe.bankAccount.validateRoutingNumber(routingNumber, country)) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Routing Number");
+                errorMessages += self.options.routingNumberLabel;
                 routingNumberElement.parents(
                     ".form-group").addClass("has-error");
                 valid = false;
@@ -125,7 +125,10 @@
     };
 
     $.fn.bank.defaults = {
-        stripePubKey: null
+        stripePubKey: null,
+        countryLabel: "Country",
+        accountNumberLabel: "Account Number",
+        routingNumberLabel: "Routing Number"
     };
 
     /** Augment a <form> to request a token from a credit card, then submit
@@ -190,6 +193,16 @@
     Card.prototype = {
         init: function () {
             var self = this;
+
+            self.stripe = Stripe(self.options.stripePubKey);
+            if( self.options.stripeIntentSecret ) {
+                var elements = self.stripe.elements();
+                self.cardElement = elements.create("card", {
+                    hidePostalCode: true
+                });
+                self.cardElement.mount("#card-element");
+            }
+
             self.element.find("#card-update").click(function(event) {
                 event.preventDefault();
                 self.showCardInputFields();
@@ -267,17 +280,23 @@
             });
         },
 
-        stripeResponseHandler: function(status, response) {
+        stripeResponseHandler: function(resp) {
             var self = this;
             var submitButton = self.element.find("[type='submit']");
-            if (response.error) {
+            if (resp.error) {
                 // show the errors on the form
-                showMessages([response.error.message], "error");
+                showMessages([resp.error.message], "error");
                 submitButton.removeAttr("disabled");
             } else {
-                // token contains id, last4, and card type
-                var token = response.id;
-                // insert the token into the form so it gets submitted to the server
+                var token = resp.id;
+                if( !token && resp.paymentIntent ) {
+                    token = resp.paymentIntent.id;
+                }
+                if( !token && resp.setupIntent ) {
+                    token = resp.setupIntent.id;
+                }
+                // insert the token into the form so it gets submitted
+                // to the server.
                 self.element.append("<input type='hidden' name='stripeToken' value='" + token + "'/>");
                 // and submit
                 self.element.get(0).submit();
@@ -304,7 +323,7 @@
             var number = numberElement.val();
             if( number === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Card Number");
+                errorMessages += self.options.cardNumberLabel;
                 numberElement.parents(".form-group").addClass("has-error");
                 valid = false;
             }
@@ -312,7 +331,7 @@
             var cvc = cvcElement.val();
             if( cvc === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Security Code");
+                errorMessages += self.options.securityCodeLabel;
                 cvcElement.parents(".form-group").addClass("has-error");
                 valid = false;
             }
@@ -322,7 +341,7 @@
             var expYear = expYearElement.val();
             if( expMonth === "" || expYear === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Expiration");
+                errorMessages += self.options.expirationLabel;
                 expMonthElement.parents(".form-group").addClass("has-error");
                 expYearElement.parents(".form-group").addClass("has-error");
                 valid = false;
@@ -333,7 +352,7 @@
             var name = nameElement.val();
             if( name === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Card Holder");
+                errorMessages += self.options.cardHolderLabel;
                 nameElement.parents(".form-group").addClass("has-error");
                 valid = false;
             }
@@ -341,7 +360,7 @@
             var addressLine1 = addressLine1Element.val();
             if( addressLine1 === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Street address");
+                errorMessages += self.options.streetAddressLabel;
                 addressLine1Element.parents(
                     ".form-group").addClass("has-error");
                 valid = false;
@@ -350,7 +369,7 @@
             var addressCity = addressCityElement.val();
             if( addressCity === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("City/Town");
+                errorMessages += self.options.localityLabel;
                 addressCityElement.parents(".form-group").addClass("has-error");
                 valid = false;
             }
@@ -358,7 +377,7 @@
             var addressState = addressStateElement.val();
             if( addressState === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("State/Province/County");
+                errorMessages += self.options.regionLabel;
                 addressStateElement.parents(
                     ".form-group").addClass("has-error");
                 valid = false;
@@ -367,7 +386,7 @@
             var addressZip = addressZipElement.val();
             if( addressZip === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Zip/Postal code");
+                errorMessages += self.options.postalCodeLabel;
                 addressZipElement.parents(".form-group").addClass("has-error");
                 valid = false;
             }
@@ -375,7 +394,7 @@
             var addressCountry = addressCountryElement.val();
             if( addressCountry === "" ) {
                 if( errorMessages ) { errorMessages += ", "; }
-                errorMessages += gettext("Country");
+                errorMessages += self.options.countryLabel;
                 addressCountryElement.parents(
                     ".form-group").addClass("has-error");
                 valid = false;
@@ -386,21 +405,70 @@
             }
             if( valid ) {
                 // this identifies your website in the createToken call below
-                Stripe.setPublishableKey(self.options.stripePubKey);
-                Stripe.createToken({
-                    number: number,
-                    cvc: cvc,
-                    exp_month: expMonth,
-                    exp_year: expYear,
-                    name: name,
-                    address_line1: addressLine1,
-                    address_city: addressCity,
-                    address_state: addressState,
-                    address_zip: addressZip,
-                    address_country: addressCountry
-                }, function(status, response) {
-                    self.stripeResponseHandler(status, response);
-                });
+                if( self.options.stripeIntentSecret ) {
+                    if( self.options.stripeIntentSecret.substring(0, 3) === 'pi_' ) {
+                        self.stripe.confirmCardPayment(
+                            self.options.stripeIntentSecret, {
+                                payment_method: {
+                                    type: "card",
+                                    card: self.cardElement,
+                                    billing_details: {
+                                        address: {
+                                            city: addressCity,
+                                            country: addressCountry,
+                                            line1: addressLine1,
+                                            // line2: null,
+                                            postal_code: addressZip,
+                                            state: addressState,
+                                        },
+                                        name: name,
+                                    }
+                                }
+                            }
+                        ).then(function(resp) {
+                            self.stripeResponseHandler(resp);
+                        });
+                    } else {
+                        self.stripe.confirmCardSetup(
+                            self.options.stripeIntentSecret, {
+                                payment_method: {
+                                    type: "card",
+                                    card: self.cardElement,
+                                    billing_details: {
+                                        address: {
+                                            city: addressCity,
+                                            country: addressCountry,
+                                            line1: addressLine1,
+                                            // line2: null,
+                                            postal_code: addressZip,
+                                            state: addressState,
+                                        },
+                                        name: name,
+                                    }
+                                }
+                            }
+                        ).then(function(resp) {
+                            self.stripeResponseHandler(resp);
+                        });
+                    }
+                } else {
+                    // use https://js.stripe.com/v2/
+                    Stripe.setPublishableKey(self.options.stripePubKey);
+                    Stripe.createToken({
+                        number: number,
+                        cvc: cvc,
+                        exp_month: expMonth,
+                        exp_year: expYear,
+                        name: name,
+                        address_line1: addressLine1,
+                        address_city: addressCity,
+                        address_state: addressState,
+                        address_zip: addressZip,
+                        address_country: addressCountry
+                    }, function(status, resp) {
+                        self.stripeResponseHandler(resp);
+                    });
+                }
             } else {
                 showMessages([errorMessages], "error");
                 submitButton.removeAttr("disabled");
@@ -417,7 +485,17 @@
 
     $.fn.card.defaults = {
         stripePubKey: null,
-        saas_api_card: null
+        stripeIntentSecret: null,
+        saas_api_card: null,
+        cardNumberLabel: "Card Number",
+        securityCodeLabel: "Security Code",
+        expirationLabel: "Expiration",
+        cardHolderLabel: "Card Holder",
+        streetAddressLabel: "Street address",
+        localityLabel: "City/Town",
+        regionLabel: "State/Province/County",
+        postalCodeLabel: "Zip/Postal code",
+        countryLabel: "Country"
     };
 
 
